/***
 * Fractal ADL Parser
 * Copyright (C) 2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.adl.arguments;

import java.util.ArrayList;
import java.util.List;

public final class ArgumentHelper {
  private ArgumentHelper() {
  }

  public static List<String> splitNameList(final String nameList) {
    return splitNameList(nameList, ",", "(", ")");
  }

  public static List<String> splitNameList(String nameList,
      final String listSeparator, final String lDelimiter,
      final String rDelimiter) {
    final List<String> l = new ArrayList<String>();

    int i, j;
    String currentName = "";
    int openedDelimiter = 0;
    while ((i = nameList.indexOf(listSeparator)) != -1) {
      int nextDelim = 0; // 0->listSeparator; 1->lDelimiter; 2->rDelimiter
      j = nameList.indexOf(lDelimiter);
      if (j != -1 && j < i) {
        nextDelim = 1;
        i = j;
      }
      j = nameList.indexOf(rDelimiter);
      if (j != -1 && j < i) {
        nextDelim = 2;
        i = j;
      }

      if (nextDelim == 0) {
        // the token is a listSeparator
        if (openedDelimiter == 0) {
          // add the current name in the list. and start a new name
          currentName += nameList.substring(0, i);
          l.add(currentName);
          currentName = "";
        } else {
          currentName += nameList.substring(0, i + listSeparator.length());
        }
        nameList = nameList.substring(i + listSeparator.length());
      } else if (nextDelim == 1) {
        // the token is a lDelimitier
        openedDelimiter++;
        currentName += nameList.substring(0, i + lDelimiter.length());
        nameList = nameList.substring(i + lDelimiter.length());
      } else if (nextDelim == 2) {
        // the token is a rDelimiter
        if (openedDelimiter == 0) {
          // parse error
          return null;
        }
        openedDelimiter--;
        currentName += nameList.substring(0, i + rDelimiter.length());
        nameList = nameList.substring(i + rDelimiter.length());
      }
    }
    l.add(currentName + nameList);
    return l;
  }

  public static String[] splitNameRef(final String line) {
    return splitNameRef(line, "${", "}");
  }

  public static String[] splitNameRef(final String line,
      final String lDelimiter, final String rDelimiter) {
    int i, j, i1;

    i1 = line.indexOf(lDelimiter);

    if (i1 == -1) return new String[]{line};

    int level = 1;

    final String[] result = new String[3];
    result[0] = line.substring(0, i1);
    i = i1 + lDelimiter.length();
    do {
      j = line.indexOf(rDelimiter, i);
      if (j == -1) {
        // syntax error
        return null;
      }

      // count opening delimiter from i to j.
      final String s = line.substring(i, j);
      int k = 0;
      int l;
      while ((l = s.indexOf(lDelimiter, k)) != -1) {
        k = l + lDelimiter.length();
        level++;
      }

      level--;
      i = j + rDelimiter.length();
      j = line.indexOf(rDelimiter, i);

    } while (level > 0 && j != -1);

    if (level != 0) {
      // syntax error
      return null;
    }
    result[1] = line.substring(i1 + lDelimiter.length(), i
        - rDelimiter.length());
    result[2] = line.substring(i + rDelimiter.length() - 1);
    return result;
  }
}
